package Centurion.lib.jazzlib;

public final class Deflater
{
    private int level;
    private boolean noHeader;
    private int state;
    private long totalOut;
    private DeflaterPending pending;
    private DeflaterEngine engine;
    
    public Deflater() {
        this(-1, false);
    }
    
    public Deflater(final int n) {
        this(n, false);
    }
    
    public Deflater(int lvl, final boolean nowrap) {
        super();
        if (lvl == -1) {
            lvl = 6;
        }
        else if (lvl < 0 || lvl > 9) {
            throw new IllegalArgumentException();
        }
        this.pending = new DeflaterPending();
        this.engine = new DeflaterEngine(this.pending);
        this.noHeader = nowrap;
        this.setStrategy(0);
        this.setLevel(lvl);
        this.reset();
    }
    
    public final void reset() {
        this.state = (this.noHeader ? 16 : 0);
        this.totalOut = 0L;
        this.pending.reset();
        this.engine.reset();
    }
    
    public final int getTotalIn() {
        return (int)this.engine.getTotalIn();
    }
    
    public final int getTotalOut() {
        return (int)this.totalOut;
    }
    
    final void flush() {
        this.state |= 0x4;
    }
    
    public final void finish() {
        this.state |= 0xC;
    }
    
    public final boolean finished() {
        return this.state == 30 && this.pending.isFlushed();
    }
    
    public final boolean needsInput() {
        return this.engine.needsInput();
    }
    
    public final void setInput(final byte[] array, final int n, final int n2) {
        if ((this.state & 0x8) != 0x0) {
            throw new IllegalStateException("finish()/end() already called");
        }
        this.engine.setInput(array, n, n2);
    }
    
    public final void setLevel(int lvl) {
        if (lvl == -1) {
            lvl = 6;
        }
        else if (lvl < 0 || lvl > 9) {
            throw new IllegalArgumentException();
        }
        if (this.level != lvl) {
            this.level = lvl;
            this.engine.setLevel(lvl);
        }
    }
    
    public final void setStrategy(final int n) {
        if (n != 0 && n != 1 && n != 2) {
            throw new IllegalArgumentException();
        }
        this.engine.setStrategy(n);
    }
    
    public final int deflate(final byte[] output, int offset, int length) {
        final int origLength = length;
        if (this.state == 127) {
            throw new IllegalStateException("Deflater closed");
        }
        if (this.state < 16) {
            int level_flags;
            if ((level_flags = this.level - 1 >> 1) < 0 || level_flags > 3) {
                level_flags = 3;
            }
            int header = 0x7800 | level_flags << 6;
            if ((this.state & 0x1) != 0x0) {
                header |= 0x20;
            }
            this.pending.writeShortMSB(header + (31 - header % 31));
            if ((this.state & 0x1) != 0x0) {
                final int chksum = this.engine.getAdler();
                this.engine.resetAdler();
                this.pending.writeShortMSB(chksum >> 16);
                this.pending.writeShortMSB(chksum & 0xFFFF);
            }
            this.state = (0x10 | (this.state & 0xC));
        }
        while (true) {
            final int count = this.pending.flush(output, offset, length);
            offset += count;
            this.totalOut += count;
            if ((length -= count) == 0 || this.state == 30) {
                return origLength - length;
            }
            if (this.engine.deflate((this.state & 0x4) != 0x0, (this.state & 0x8) != 0x0)) {
                continue;
            }
            if (this.state == 16) {
                return origLength - length;
            }
            if (this.state == 20) {
                if (this.level != 0) {
                    for (int i = 8 + (-this.pending.getBitCount() & 0x7); i > 0; i -= 10) {
                        this.pending.writeBits(2, 10);
                    }
                }
                this.state = 16;
            }
            else {
                if (this.state != 28) {
                    continue;
                }
                this.pending.alignToByte();
                if (!this.noHeader) {
                    final int adler = this.engine.getAdler();
                    this.pending.writeShortMSB(adler >> 16);
                    this.pending.writeShortMSB(adler & 0xFFFF);
                }
                this.state = 30;
            }
        }
    }
}
