package Centurion.lib.jazzlib;

import java.io.*;
import java.util.*;

public final class ZipFile
{
    private byte[] b;
    private int off;
    private int length;
    private Hashtable entries;
    
    public ZipFile(final InputStream inputStream) throws ZipException, IOException {
        super();
        final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        final byte[] array = new byte[1024];
        int read;
        while ((read = inputStream.read(array)) > 0) {
            byteArrayOutputStream.write(array, 0, read);
        }
        this.b = byteArrayOutputStream.toByteArray();
        this.off = 0;
        this.length = this.b.length;
        this.check();
    }
    
    private void check() throws ZipException {
        if (this.length < 4) {
            throw new ZipException("Not a valid zip archive");
        }
        if (((this.b[this.off] & 0xFF) | (this.b[this.off + 1] & 0xFF) << 8 | (this.b[this.off + 2] & 0xFF) << 16 | (this.b[this.off + 3] & 0xFF) << 24) != 0x4034B50L) {
            throw new ZipException("Not a valid zip archive");
        }
    }
    
    private void readEntries() throws ZipException, IOException {
        final RandomAccessFile raf = new RandomAccessFile(this.b, this.off, this.length);
        int pos = this.length - 22;
        while (pos >= Math.max(0, pos - 65536)) {
            raf.seek(this.off + pos--);
            if (raf.readInt() == 101010256L) {
                if (raf.skip(6L) != 6L) {
                    throw new EOFException();
                }
                final int count = raf.readShort();
                if (raf.skip(4L) != 4L) {
                    throw new EOFException();
                }
                final int centralOffset = raf.readInt();
                this.entries = new Hashtable(count + count / 2);
                raf.seek(this.off + centralOffset);
                for (int j = 0; j < count; ++j) {
                    if (raf.readInt() != 33639248L) {
                        throw new ZipException("Wrong Central Directory signature");
                    }
                    raf.skip(6L);
                    final int method = raf.readShort();
                    final int dostime = raf.readInt();
                    final int method918 = raf.readInt();
                    final int method919 = raf.readInt();
                    final int method920 = raf.readInt();
                    final int nameLen = raf.readShort();
                    final int extraLen = raf.readShort();
                    final int commentLen = raf.readShort();
                    raf.skip(8L);
                    final int offset = raf.readInt();
                    final String name = raf.readUTF(nameLen);
                    final ZipEntry entry;
                    (entry = new ZipEntry(name)).setMethod(method);
                    entry.setCrc((long)method918 & 0xFFFFFFFFL);
                    entry.setSize((long)method920 & 0xFFFFFFFFL);
                    entry.setCompressedSize((long)method919 & 0xFFFFFFFFL);
                    entry.setDOSTime(dostime);
                    if (extraLen > 0) {
                        final byte[] array = new byte[extraLen];
                        raf.readFully(array);
                        entry.setExtra(array);
                    }
                    if (commentLen > 0) {
                        entry.setComment(raf.readUTF(commentLen));
                    }
                    entry.offset = offset;
                    this.entries.put(name, entry);
                }
                return;
            }
        }
        throw new ZipException("central directory not found, probably not a zip archive");
    }
    
    public final Enumeration entries() {
        try {
            return this.getEntries().elements();
        }
        catch (IOException ex) {
            return new Hashtable().elements();
        }
    }
    
    private Hashtable getEntries() throws IOException {
        if (this.entries == null) {
            this.readEntries();
        }
        return this.entries;
    }
}
